package com.redhat.thermostat.tools.eclipse.plugin.wizards;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import com.redhat.thermostat.tools.eclipse.plugin.Activator;
import com.redhat.thermostat.tools.eclipse.plugin.Messages;
import com.redhat.thermostat.tools.eclipse.plugin.wizards.PluginXmlCreator.Decisions;

/**
 * Wraps a {@link PluginXmlCreator} with GUI logic (including
 * progress notification and prompts).
 */
public class GuiPluginXmlCreator {

    public boolean createPluginWithFeedback(final String containerName, final Wizard wizard) {
        IRunnableWithProgress op = new IRunnableWithProgress() {
            public void run(IProgressMonitor monitor) throws InvocationTargetException {
                try {
                    create(containerName, monitor, wizard.getShell());
                } catch (CoreException e) {
                    throw new InvocationTargetException(e);
                } finally {
                    monitor.done();
                }
            }
        };
        try {
            wizard.getContainer().run(true, false, op);
        } catch (InterruptedException e) {
            return false;
        } catch (InvocationTargetException e) {
            Throwable realException = e.getTargetException();
            MessageDialog.openError(wizard.getShell(), Messages.GuiPluginXmlCreator_error,
                    realException.getMessage());
            return false;
        }
        return true;
    }

    private void create(String containerName, IProgressMonitor monitor, final Shell shell) throws CoreException {

        monitor.beginTask(Messages.GuiPluginXmlCreator_creatingFile, 2);
        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
        PluginXmlCreator creator = new PluginXmlCreator(root);

        try {
            final IFile file = creator.create(containerName, monitor, new Decisions() {
                @Override
                public boolean overwriteExistingFile() {
                    final boolean[] overwrite = new boolean [1];
                    shell.getDisplay().syncExec(new Runnable() {
                        @Override
                        public void run() {
                            overwrite[0] = MessageDialog.openConfirm(shell, Messages.GuiPluginXmlCreator_confirmOverwriteTitle, Messages.PluginXmlCreationWizard_confirmOverwriteMessage);
                        }
                    });
                    return overwrite[0];
                }
            });
            monitor.worked(1);
            monitor.setTaskName(Messages.GuiPluginXmlCreator_openingFileForEditing);
            shell.getDisplay().asyncExec(new Runnable() {
                public void run() {
                    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
                    try {
                        IDE.openEditor(page, file, true);
                    } catch (PartInitException e) {
                        e.printStackTrace();
                    }
                }
            });
            monitor.worked(1);
        } catch (IOException e) {
            throwCoreException(Messages.GuiPluginXmlCreator_errorWritingFile, e);
        }
    }

    private void throwCoreException(String message, Throwable cause) throws CoreException {
        IStatus status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, IStatus.OK, message, cause);
        throw new CoreException(status);
    }
}
